/* ----------------------------------------------------------------------
 * debug.c
 *
 * MILO debugger/monitor main program
 * Written by Andreas Busse
 *
 * This file is subject to the terms and conditions of the
 * GNU General Public License. See the file COPYING in the
 * main directory of this archive for more details.
 * ---------------------------------------------------------------------- */

#include <stand/debug.h>

#ifdef STANDALONE
#include <stdio.h>
#else
#include <stddef.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/file.h>
#include <sys/types.h>
#include <arc/arc.h>
#endif

/* ----------------------------------------------------------------------
 * Local Prototypes
 * ---------------------------------------------------------------------- */

int main(int argc, char **argv);
int debug(void);
int nop(void);
int help(void);
void prompt(void);


/* Command table */

static struct cmdent
{
  char	*cmd_name;
  int	(*cmd_fctn)();
} cmdtbl[] =

{
  CMD_DUMP, dump,
  CMD_FILL, fill,
  CMD_HELP, help,
  CMD_NOP, nop,			/* must be the last command */
};

#define NUMCMDS 3

/* ----------------------------------------------------------------------
 * main()
 * (Used only for standalone version)
 * ---------------------------------------------------------------------- */
#ifdef STANDALONE
main(argc,argv)
     int argc;
     char **argv;
{
  debug();
}
#endif

/* ----------------------------------------------------------------------
 * debug()
 * Debugger/Monitor main loop
 * ---------------------------------------------------------------------- */
int debug()
{
  char	line[MAXBUFLEN];
  char	**argp = NULL;
  char	*cp;
  int	argnum, i;

  /*
   * Now get commands one at a time and execute the
   * appropriate routine to deal with them.
   */

  printf("\r\n\r\nMILO built-in Debugger/Monitor (quit to continue boot process)\r\n\r\n");
  
  for (;;)
    {
      prompt();

#ifdef STANDALONE      
      if (fgets(line, sizeof(line), stdin) == NULL)
#else
      if (gets(line) == NULL)
#endif	  
	break;

      /* Strip trailing CR-LF */

      cp = line + strlen(line) - 1;
      while (cp >= line && (*cp == '\n' || *cp == '\r'))
	*cp-- = '\0';

      /* Null command */
		
      if ((argnum = parsit(line, &argp)) == 0)
	continue;

      /* a motion to adjourn is always in order */

      if (!strcasecmp(argp[0], "quit"))
	break;

      for (i = 0; i < NUMCMDS; ++i)
	if (!strcasecmp(cmdtbl[i].cmd_name, argp[0]))
	  break;

      if (i < NUMCMDS)
	{
	  (*cmdtbl[i].cmd_fctn)(argnum, argp);
	}
      else
	{
	  printf("Unrecognized command.\r\n");
	  (void) fflush(stdout);
	}
    }

  return(0);
}

/* ----------------------------------------------------------------------
 * nop()
 * NOP Function, only for debugging purposes.
 * ---------------------------------------------------------------------- */
int nop()
{
  return(0);
}

/* ----------------------------------------------------------------------
 * help()
 * Print info about available commands
 * ---------------------------------------------------------------------- */
int help()
{
  printf("\r\nKnown commands:\r\n\r\n");
  printf("dump [-b|h|w] start [end]\t\tdump memory contents\r\n");
  printf("fill [-b|h|w] start [end] val\t\tfill memory with value\r\n");
  printf("quit\t\t\t\t\tquit debugger\r\n");
  printf("\r\n\r\n");
  return(0);
}

/* ----------------------------------------------------------------------
 * prompt()
 * Prompt for a new command
 * ---------------------------------------------------------------------- */
void prompt()
{
  printf("> ");
  fflush(stdout);
}

